{ copyright  Kestral Computing
  refer to xyGraph.pas, index.htm for more information }

unit xyHolder;

interface

uses
  Wintypes, winprocs, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, grids, clipbrd;

const tabchar = #9;

type
  TPaddingPolicy = (ppTotal, ppExtra);

  TxyHolder = class(TComponent)
  private
    { Private declarations }
    FData:TStringlist;
    FOnChange:TNotifyEvent;
    FID:string;
    FTextseparators, FTextMarkers:string;
    FXPaddingPolicy,FYPaddingPolicy:TPaddingPolicy;
    FXPadCount,FYPadCount:integer;

    Function GetSeparators:string;
    procedure SetSeparators(s:string);
    procedure SetData(v:Tstringlist);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);         override;
    destructor destroy;                            override;

    procedure Clear;

    procedure WriteToFile(const filename:string);
    procedure WriteToStream(stream:TStream);
    procedure WriteToStringGrid(sgrid:TStringgrid);
    procedure CopytoClipboard;

    procedure ReadFromFile(const filename:string);
    procedure ReadFromStream(stream:TStream);
    procedure ReadFromStringGrid(sgrid:TStringGrid);
    procedure PasteFromClipboard;

  published
    { Published declarations }
    property Textseparators:string read Getseparators write Setseparators;
    property TextMarkers:string read FTextMarkers write FTextMarkers;
    property data:TStringList read FData write SetData;

    property XPaddingPolicy:TPaddingPolicy read FXPaddingPolicy write FXPaddingPolicy;
    property YPaddingPolicy:TPaddingPolicy read FYPaddingPolicy write FYPaddingPolicy;
    property XPadCount:integer read FXPadCount write FXPadCount;
    property YPadCount:integer read FYPadCount write FYPadCount;
    property ID:String read FID write FID;

    property OnChange:TNotifyEvent read FOnChange write FOnChange;

  end;

function FindxyHolder(const xyname:string):TxyHolder;

implementation

uses xyData;

constructor TxyHolder.Create(AOwner:TComponent);
begin
  inherited create(AOwner);
  FData := TStringlist.create;
  FXPaddingPolicy := ppExtra;
  FYPaddingPolicy := ppExtra;
  FXPadCount := 2;
  FYPadCount := 10;
  FTextSeparators := #9+',';
  FTextMarkers := '''"';
end;

destructor TxyHolder.destroy;
begin
  FData.Free;
  inherited destroy;
end;

procedure TxyHolder.clear;
begin
  FData.Clear;
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TxyHolder.WriteToFile(const filename:string);
begin
  FData.SaveToFile(Filename);
end;

procedure TxyHolder.SetSeparators;
var i:integer;
begin
  i := pos('T', s);
  if i > 0 then s[i] := #9;
  FTextSeparators := s;
end;

function  TxyHolder.GetSeparators;
var i:integer;
begin
  result := FTextSeparators;
  i := pos(#9, result);
  if i > 0 then result[i] := 'T';
end;

procedure TxyHolder.SetData;
begin
  FData.Assign(v);
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TxyHolder.WriteToStream(stream:TStream);
begin
  FData.SaveToStream(stream);
end;

procedure TxyHolder.WriteToStringGrid(sgrid:TStringgrid);
var i,j,rows,numcols:integer;
    s,s2:string;
    ended:boolean;
begin
  rows := FData.count;
  for i := sgrid.ColCount - 1 downto 0 do sgrid.cols[i].clear;
  sgrid.Colcount := 0;
  sgrid.rowCount := rows;
  for i := 0 to FData.Count - 1 do
    begin
    j := 0;
    s := FData[i];
    s2 := cellFromStr(s, FTextSeparators,FTextMarkers, j, ended);
    while not ended do
      begin
      if (sgrid.Colcount < j+1) then sgrid.Colcount := j+1;
      sgrid.Cells[j,i] := s2;
      inc(j);
      s2 := cellFromStr(s, FTextSeparators,FTextMarkers, j, ended);
      end;
    end;

   if FXPaddingPolicy = ppTotal then
     if sgrid.ColCount < FXPadCount then
       sgrid.Colcount := FXpadCount
     else
   else
     sgrid.Colcount := SGrid.Colcount + FXPadCount;
   if FYPaddingPolicy = ppTotal then
     if sgrid.RowCount < FYPadCount then
       sgrid.Rowcount := FYPadCount
     else
   else
     sgrid.Rowcount := SGrid.Rowcount + FYPadCount;
end;

procedure TxyHolder.CopytoClipboard;
var h:Thandle;
    p,p1:PChar;
    strm:TMemoryStream;
begin
  p1 := FData.GetText;
  h := GlobalAlloc(GMEM_Moveable, strlen(p1)+1);
  try
    p := GlobalLock(h);
    try
      strcopy(p,p1);
      clipboard.open;
      Clipboard.SetAsHandle(CF_Text, h);
      Clipboard.Close;
    finally
      GlobalUnLock(h);
    end;
  except
    Globalfree(h);
    raise;
  end;
end;

procedure TxyHolder.ReadFromFile(const filename:string);
begin
  FData.LoadFromFile(filename);
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TxyHolder.ReadFromStream(stream:TStream);
begin
  FData.LoadFromStream(stream);
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TxyHolder.ReadFromStringGrid(sgrid:TStringGrid);
var i,j:word;
    s:string;
    function wrap(const s:string):string;
    var i:integer;
        wrap:boolean;
    begin
      wrap := false;
      for i := 1 to length(FTextseparators) do
         if (pos(FTextSeparators[i],s) > 0) or (pos(FTextMarkers[i],s) > 0) then wrap := true;
      if wrap or (pos(' ',s) > 0) then
        begin
        result := FTextMarkers[1];
        for i := 1 to length(s) do
          if pos(s[i], FTextMarkers)>0 then
            result := result + s[i] + s[i]
          else
            result := result + s[i];
        result := result + FTextMarkers[1];
        end
      else
        result := s;
    end;
begin
  if length(FTextSeparators) < 1 then
     raise exception.create('No Text Separators supplied');
  FData.clear;
  for i := 0 to sgrid.rowcount-1 do
    begin
    s := '';
    for j := 0 to sgrid.colcount-1 do
      begin
      if j <> 0 then s := s+FTextSeparators[1];
      s := s + wrap(sgrid.cells[j,i]);
      end;
    FData.add(s);
    end;
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TxyHolder.PasteFromClipboard;
var h:Thandle;
    p:PChar;
    strm:TMemoryStream;
begin
  h := ClipBoard.GetAsHandle(CF_Text);
  try
    if h = 0 then exit;
    p := GlobalLock(h);
    try
      FData.SetText(p);
      if pos(#9, FTextSeparators) = 0 then FTextSeparators := FTextSeparators + #9;
    finally
      GlobalunLock(h);
    end;
  except
    GlobalFree(h);
    raise;
  end;
  if Assigned(FOnChange) then FOnChange(self);
end;

function FindxyHolder(const xyname:string):TxyHolder;
var iForm, iComp:integer;
begin
  { look through all forms for any xyDataHolder with a name matching name }
  result := nil;
  {$IFDEF WIN32}
  //both 32bit delphi's have datamodules as well. check these first
  iForm := 0;
  while (result = nil) and (iForm < screen.DataModulecount) do
    begin
    with screen.datamodules[iForm] do
      begin
      iComp := 0;
      while (result = nil) and (iComp < ComponentCount) do
        begin
        if (Components[iComp] is TxyHolder)
              and ((Components[iComp] As TxyHolder).ID = xyName) then
             result := (Components[iComp] As TxyHolder);
        inc(iComp);
        end;
      end;
    inc(iForm);
    end;
  {$ENDIF}
  iForm := 0;
  while (result = nil) and (iForm < screen.Formcount) do
    begin
    with screen.Forms[iForm] do
      begin
      iComp := 0;
      while (result = nil) and (iComp < ComponentCount) do
        begin
        if (Components[iComp] is TxyHolder) and
           ((Components[iComp] As TxyHolder).ID = xyName) then
             result := (Components[iComp] As TxyHolder);
        inc(iComp);
        end;
      end;
    inc(iForm);
    end;
end;

end.
